@@ -37,10 +37,13 @@ import (
 	apierrors "github.com/openshift/cluster-api/pkg/errors"
 	providerconfigv1 "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsproviderconfig/v1beta1"
 
+	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/ec2"
 
 	awsclient "sigs.k8s.io/cluster-api-provider-aws/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client"
+
+	machinecontroller "github.com/openshift/cluster-api/pkg/controller/machine"
 )
 
 const (
@@ -101,29 +104,113 @@ func (a *Actuator) handleMachineError(machine *machinev1.Machine, err *apierrors
 		a.eventRecorder.Eventf(machine, corev1.EventTypeWarning, "Failed"+eventAction, "%v", err.Reason)
 	}
 
-	glog.Errorf("Machine error: %v", err.Message)
+	glog.Errorf("%s: Machine error: %v", machine.Name, err.Message)
 	return err
 }
 
 // Create runs a new EC2 instance
 func (a *Actuator) Create(context context.Context, cluster *clusterv1.Cluster, machine *machinev1.Machine) error {
-	glog.Info("creating machine")
+	glog.Infof("%s: creating machine", machine.Name)
 	instance, err := a.CreateMachine(cluster, machine)
 	if err != nil {
-		glog.Errorf("error creating machine: %v", err)
+		glog.Errorf("%s: error creating machine: %v", machine.Name, err)
 		updateConditionError := a.updateMachineProviderConditions(machine, providerconfigv1.MachineCreation, MachineCreationFailed, err.Error())
 		if updateConditionError != nil {
-			glog.Errorf("error updating machine conditions: %v", updateConditionError)
+			glog.Errorf("%s: error updating machine conditions: %v", machine.Name, updateConditionError)
 		}
 		return err
 	}
-	return a.updateStatus(machine, instance)
+	updatedMachine, err := a.updateProviderID(machine, instance)
+	if err != nil {
+		return fmt.Errorf("%s: failed to update machine object with providerID: %v", machine.Name, err)
+	}
+	machine = updatedMachine
+
+	updatedMachine, err = a.setMachineCloudProviderSpecifics(machine, instance)
+	if err != nil {
+		return fmt.Errorf("%s: failed to set machine cloud provider specifics: %v", machine.Name, err)
+	}
+
+	return a.updateStatus(updatedMachine, instance)
+}
+
+// updateProviderID adds providerID in the machine spec
+func (a *Actuator) setMachineCloudProviderSpecifics(machine *machinev1.Machine, instance *ec2.Instance) (*machinev1.Machine, error) {
+	if instance == nil {
+		return machine, nil
+	}
+	machineCopy := machine.DeepCopy()
+
+	if machineCopy.Labels == nil {
+		machineCopy.Labels = make(map[string]string)
+	}
+
+	if machineCopy.Annotations == nil {
+		machineCopy.Annotations = make(map[string]string)
+	}
+
+	// Reaching to machine provider config since the region is not directly
+	// providing by *ec2.Instance object
+	machineProviderConfig, err := providerConfigFromMachine(machine, a.codec)
+	if err != nil {
+		return nil, fmt.Errorf("error decoding MachineProviderConfig: %v", err)
+	}
+
+	machineCopy.Labels[machinecontroller.MachineRegionLabelName] = machineProviderConfig.Placement.Region
+
+	if instance.Placement != nil {
+		machineCopy.Labels[machinecontroller.MachineAZLabelName] = aws.StringValue(instance.Placement.AvailabilityZone)
+	}
+
+	if instance.InstanceType != nil {
+		machineCopy.Labels[machinecontroller.MachineInstanceTypeLabelName] = aws.StringValue(instance.InstanceType)
+	}
+
+	if instance.State != nil && instance.State.Name != nil {
+		machineCopy.Annotations[machinecontroller.MachineInstanceStateAnnotationName] = aws.StringValue(instance.State.Name)
+	}
+
+	if err := a.client.Update(context.Background(), machineCopy); err != nil {
+		return nil, fmt.Errorf("%s: error updating machine spec: %v", machine.Name, err)
+	}
+
+	return machineCopy, nil
+}
+
+// updateProviderID adds providerID in the machine spec
+func (a *Actuator) updateProviderID(machine *machinev1.Machine, instance *ec2.Instance) (*machinev1.Machine, error) {
+	existingProviderID := machine.Spec.ProviderID
+	machineCopy := machine.DeepCopy()
+	if instance != nil {
+		availabilityZone := ""
+		if instance.Placement != nil {
+			availabilityZone = aws.StringValue(instance.Placement.AvailabilityZone)
+		}
+		providerID := fmt.Sprintf("aws:///%s/%s", availabilityZone, aws.StringValue(instance.InstanceId))
+
+		if existingProviderID != nil && *existingProviderID == providerID {
+			glog.Infof("%s: ProviderID already set in the machine Spec with value:%s", machine.Name, *existingProviderID)
+			return machine, nil
+		}
+		machineCopy.Spec.ProviderID = &providerID
+		if err := a.client.Update(context.Background(), machineCopy); err != nil {
+			return nil, fmt.Errorf("%s: error updating machine spec ProviderID: %v", machine.Name, err)
+		}
+		glog.Infof("%s: ProviderID updated at machine spec: %s", machine.Name, providerID)
+	} else {
+		machineCopy.Spec.ProviderID = nil
+		if err := a.client.Update(context.Background(), machineCopy); err != nil {
+			return nil, fmt.Errorf("%s: error updating ProviderID in machine spec: %v", machine.Name, err)
+		}
+		glog.Infof("%s: No instance found so clearing ProviderID field in the machine spec", machine.Name)
+	}
+	return machineCopy, nil
 }
 
 func (a *Actuator) updateMachineStatus(machine *machinev1.Machine, awsStatus *providerconfigv1.AWSMachineProviderStatus, networkAddresses []corev1.NodeAddress) error {
 	awsStatusRaw, err := a.codec.EncodeProviderStatus(awsStatus)
 	if err != nil {
-		glog.Errorf("error encoding AWS provider status: %v", err)
+		glog.Errorf("%s: error encoding AWS provider status: %v", machine.Name, err)
 		return err
 	}
 
@@ -135,22 +222,22 @@ func (a *Actuator) updateMachineStatus(machine *machinev1.Machine, awsStatus *pr
 
 	oldAWSStatus := &providerconfigv1.AWSMachineProviderStatus{}
 	if err := a.codec.DecodeProviderStatus(machine.Status.ProviderStatus, oldAWSStatus); err != nil {
-		glog.Errorf("error updating machine status: %v", err)
+		glog.Errorf("%s: error updating machine status: %v", machine.Name, err)
 		return err
 	}
 
 	// TODO(vikasc): Revisit to compare complete machine status objects
 	if !equality.Semantic.DeepEqual(awsStatus, oldAWSStatus) || !equality.Semantic.DeepEqual(machine.Status.Addresses, machineCopy.Status.Addresses) {
-		glog.Infof("machine status has changed, updating")
+		glog.Infof("%s: machine status has changed, updating", machine.Name)
 		time := metav1.Now()
 		machineCopy.Status.LastUpdated = &time
 
 		if err := a.client.Status().Update(context.Background(), machineCopy); err != nil {
-			glog.Errorf("error updating machine status: %v", err)
+			glog.Errorf("%s: error updating machine status: %v", machine.Name, err)
 			return err
 		}
 	} else {
-		glog.Info("status unchanged")
+		glog.Infof("%s: status unchanged", machine.Name)
 	}
 
 	return nil
@@ -159,11 +246,11 @@ func (a *Actuator) updateMachineStatus(machine *machinev1.Machine, awsStatus *pr
 // updateMachineProviderConditions updates conditions set within machine provider status.
 func (a *Actuator) updateMachineProviderConditions(machine *machinev1.Machine, conditionType providerconfigv1.AWSMachineProviderConditionType, reason string, msg string) error {
 
-	glog.Info("updating machine conditions")
+	glog.Infof("%s: updating machine conditions", machine.Name)
 
 	awsStatus := &providerconfigv1.AWSMachineProviderStatus{}
 	if err := a.codec.DecodeProviderStatus(machine.Status.ProviderStatus, awsStatus); err != nil {
-		glog.Errorf("error decoding machine provider status: %v", err)
+		glog.Errorf("%s: error decoding machine provider status: %v", machine.Name, err)
 		return err
 	}
 
@@ -189,7 +276,7 @@ func (a *Actuator) CreateMachine(cluster *clusterv1.Cluster, machine *machinev1.
 	}
 	awsClient, err := a.awsClientBuilder(a.client, credentialsSecretName, machine.Namespace, machineProviderConfig.Placement.Region)
 	if err != nil {
-		glog.Errorf("unable to obtain AWS client: %v", err)
+		glog.Errorf("%s: unable to obtain AWS client: %v", machine.Name, err)
 		return nil, a.handleMachineError(machine, apierrors.CreateMachine("error creating aws services: %v", err), createEventAction)
 	}
 
@@ -200,14 +287,15 @@ func (a *Actuator) CreateMachine(cluster *clusterv1.Cluster, machine *machinev1.
 	// So we can safely continue to create a new machine since in the worst case
 	// we just don't delete any stopped machine.
 	if err != nil {
-		klog.Errorf("Error determining if machine is master: %v", err)
+		klog.Errorf("%s: Error determining if machine is master: %v", machine.Name, err)
 	} else {
 		if !isMaster {
 			// Prevent having a lot of stopped nodes sitting around.
 			err = removeStoppedMachine(machine, awsClient)
 			if err != nil {
-				glog.Errorf("unable to remove stopped machines: %v", err)
-				return nil, fmt.Errorf("unable to remove stopped nodes: %v", err)
+				errMsg := fmt.Sprintf("%s: unable to remove stopped machines: %v", machine.Name, err)
+				glog.Errorf(errMsg)
+				return nil, fmt.Errorf(errMsg)
 			}
 		}
 	}
@@ -222,7 +310,7 @@ func (a *Actuator) CreateMachine(cluster *clusterv1.Cluster, machine *machinev1.
 		if data, exists := userDataSecret.Data[userDataSecretKey]; exists {
 			userData = data
 		} else {
-			glog.Warningf("Secret %v/%v does not have %q field set. Thus, no user data applied when creating an instance.", machine.Namespace, machineProviderConfig.UserDataSecret.Name, userDataSecretKey)
+			glog.Warningf("%s: Secret %v/%v does not have %q field set. Thus, no user data applied when creating an instance.", machine.Name, machine.Namespace, machineProviderConfig.UserDataSecret.Name, userDataSecretKey)
 		}
 	}
 
@@ -231,7 +319,7 @@ func (a *Actuator) CreateMachine(cluster *clusterv1.Cluster, machine *machinev1.
 		return nil, a.handleMachineError(machine, apierrors.CreateMachine("error launching instance: %v", err), createEventAction)
 	}
 
-	err = a.updateLoadBalancers(awsClient, machineProviderConfig, instance)
+	err = a.updateLoadBalancers(awsClient, machineProviderConfig, instance, machine.Name)
 	if err != nil {
 		return nil, a.handleMachineError(machine, apierrors.CreateMachine("error updating load balancers: %v", err), createEventAction)
 	}
@@ -242,9 +330,9 @@ func (a *Actuator) CreateMachine(cluster *clusterv1.Cluster, machine *machinev1.
 
 // Delete deletes a machine and updates its finalizer
 func (a *Actuator) Delete(context context.Context, cluster *clusterv1.Cluster, machine *machinev1.Machine) error {
-	glog.Info("deleting machine")
+	glog.Infof("%s: deleting machine", machine.Name)
 	if err := a.DeleteMachine(cluster, machine); err != nil {
-		glog.Errorf("error deleting machine: %v", err)
+		glog.Errorf("%s: error deleting machine: %v", machine.Name, err)
 		return err
 	}
 	return nil
@@ -274,25 +362,34 @@ func (a *Actuator) DeleteMachine(cluster *clusterv1.Cluster, machine *machinev1.
 	}
 	client, err := a.awsClientBuilder(a.client, credentialsSecretName, machine.Namespace, region)
 	if err != nil {
-		errMsg := fmt.Errorf("error getting EC2 client: %v", err)
+		errMsg := fmt.Errorf("%s: error getting EC2 client: %v", machine.Name, err)
 		glog.Error(errMsg)
 		return errMsg
 	}
 
 	instances, err := getRunningInstances(machine, client)
 	if err != nil {
-		glog.Errorf("error getting running instances: %v", err)
+		glog.Errorf("%s: error getting running instances: %v", machine.Name, err)
 		return err
 	}
 	if len(instances) == 0 {
-		glog.Warningf("no instances found to delete for machine")
+		glog.Warningf("%s: no instances found to delete for machine", machine.Name)
 		return nil
 	}
 
-	err = terminateInstances(client, instances)
+	terminatingInstances, err := terminateInstances(client, instances)
 	if err != nil {
 		return a.handleMachineError(machine, apierrors.DeleteMachine(err.Error()), noEventAction)
 	}
+
+	if len(terminatingInstances) == 1 {
+		if terminatingInstances[0] != nil && terminatingInstances[0].CurrentState != nil && terminatingInstances[0].CurrentState.Name != nil {
+			machineCopy := machine.DeepCopy()
+			machineCopy.Annotations[machinecontroller.MachineInstanceStateAnnotationName] = aws.StringValue(terminatingInstances[0].CurrentState.Name)
+			a.client.Update(context.Background(), machineCopy)
+		}
+	}
+
 	a.eventRecorder.Eventf(machine, corev1.EventTypeNormal, "Deleted", "Deleted machine %v", machine.Name)
 
 	return nil
@@ -302,7 +399,7 @@ func (a *Actuator) DeleteMachine(cluster *clusterv1.Cluster, machine *machinev1.
 // for details that may have changed. (IPs and hostnames) We do not currently support making any
 // changes to actual machines in AWS. Instead these will be replaced via MachineDeployments.
 func (a *Actuator) Update(context context.Context, cluster *clusterv1.Cluster, machine *machinev1.Machine) error {
-	glog.Info("updating machine")
+	glog.Infof("%s: updating machine", machine.Name)
 
 	machineProviderConfig, err := providerConfigFromMachine(machine, a.codec)
 	if err != nil {
@@ -310,30 +407,30 @@ func (a *Actuator) Update(context context.Context, cluster *clusterv1.Cluster, m
 	}
 
 	region := machineProviderConfig.Placement.Region
-	glog.Info("obtaining EC2 client for region")
+	glog.Infof("%s: obtaining EC2 client for region", machine.Name)
 	credentialsSecretName := ""
 	if machineProviderConfig.CredentialsSecret != nil {
 		credentialsSecretName = machineProviderConfig.CredentialsSecret.Name
 	}
 	client, err := a.awsClientBuilder(a.client, credentialsSecretName, machine.Namespace, region)
 	if err != nil {
-		errMsg := fmt.Errorf("error getting EC2 client: %v", err)
+		errMsg := fmt.Errorf("%s: error getting EC2 client: %v", machine.Name, err)
 		glog.Error(errMsg)
 		return errMsg
 	}
 	// Get all instances not terminated.
 	existingInstances, err := getExistingInstances(machine, client)
 	if err != nil {
-		glog.Errorf("error getting existing instances: %v", err)
+		glog.Errorf("%s: error getting existing instances: %v", machine.Name, err)
 		return err
 	}
 	existingLen := len(existingInstances)
-	glog.Infof("found %d existing instances for machine", existingLen)
+	glog.Infof("%s: found %d existing instances for machine", machine.Name, existingLen)
 
 	// Parent controller should prevent this from ever happening by calling Exists and then Create,
 	// but instance could be deleted between the two calls.
 	if existingLen == 0 {
-		glog.Warningf("attempted to update machine but no instances found")
+		glog.Warningf("%s: attempted to update machine but no instances found", machine.Name)
 
 		a.handleMachineError(machine, apierrors.UpdateMachine("no instance found, reason unknown"), updateEventAction)
 
@@ -352,10 +449,10 @@ func (a *Actuator) Update(context context.Context, cluster *clusterv1.Cluster, m
 	if runningLen > 0 {
 		// It would be very unusual to have more than one here, but it is
 		// possible if someone manually provisions a machine with same tag name.
-		glog.Infof("found %d running instances for machine %v", runningLen, machine.Name)
+		glog.Infof("%s: found %d running instances for machine", machine.Name, runningLen)
 		newestInstance = runningInstances[0]
 
-		err = a.updateLoadBalancers(client, machineProviderConfig, newestInstance)
+		err = a.updateLoadBalancers(client, machineProviderConfig, newestInstance, machine.Name)
 		if err != nil {
 			a.handleMachineError(machine, apierrors.CreateMachine("Error updating load balancers: %v", err), updateEventAction)
 			return err
@@ -368,6 +465,18 @@ func (a *Actuator) Update(context context.Context, cluster *clusterv1.Cluster, m
 
 	a.eventRecorder.Eventf(machine, corev1.EventTypeNormal, "Updated", "Updated machine %v", machine.Name)
 
+	modMachine, err := a.setMachineCloudProviderSpecifics(machine, newestInstance)
+	if err != nil {
+		return fmt.Errorf("%s: failed to set machine cloud provider specifics: %v", machine.Name, err)
+	}
+	machine = modMachine
+
+	updatedMachine, err := a.updateProviderID(machine, newestInstance)
+	if err != nil {
+		return fmt.Errorf("%s: failed to update machine object with providerID: %v", machine.Name, err)
+	}
+	machine = updatedMachine
+
 	// We do not support making changes to pre-existing instances, just update status.
 	return a.updateStatus(machine, newestInstance)
 }
@@ -375,34 +484,34 @@ func (a *Actuator) Update(context context.Context, cluster *clusterv1.Cluster, m
 // Exists determines if the given machine currently exists. For AWS we query for instances in
 // running state, with a matching name tag, to determine a match.
 func (a *Actuator) Exists(context context.Context, cluster *clusterv1.Cluster, machine *machinev1.Machine) (bool, error) {
-	glog.Info("Checking if machine exists")
+	glog.Infof("%s: Checking if machine exists", machine.Name)
 
 	instances, err := a.getMachineInstances(cluster, machine)
 	if err != nil {
-		glog.Errorf("Error getting running instances: %v", err)
+		glog.Errorf("%s: Error getting running instances: %v", machine.Name, err)
 		return false, err
 	}
 	if len(instances) == 0 {
-		glog.Info("Instance does not exist")
+		glog.Infof("%s: Instance does not exist", machine.Name)
 		return false, nil
 	}
 
 	// If more than one result was returned, it will be handled in Update.
-	glog.Infof("Instance exists as %q", *instances[0].InstanceId)
+	glog.Infof("%s: Instance exists as %q", machine.Name, *instances[0].InstanceId)
 	return true, nil
 }
 
 // Describe provides information about machine's instance(s)
 func (a *Actuator) Describe(cluster *clusterv1.Cluster, machine *machinev1.Machine) (*ec2.Instance, error) {
-	glog.Infof("Checking if machine exists")
+	glog.Infof("%s: Checking if machine exists", machine.Name)
 
 	instances, err := a.getMachineInstances(cluster, machine)
 	if err != nil {
-		glog.Errorf("Error getting running instances: %v", err)
+		glog.Errorf("%s: Error getting running instances: %v", machine.Name, err)
 		return nil, err
 	}
 	if len(instances) == 0 {
-		glog.Info("Instance does not exist")
+		glog.Infof("%s: Instance does not exist", machine.Name)
 		return nil, nil
 	}
 
@@ -412,7 +521,7 @@ func (a *Actuator) Describe(cluster *clusterv1.Cluster, machine *machinev1.Machi
 func (a *Actuator) getMachineInstances(cluster *clusterv1.Cluster, machine *machinev1.Machine) ([]*ec2.Instance, error) {
 	machineProviderConfig, err := providerConfigFromMachine(machine, a.codec)
 	if err != nil {
-		glog.Errorf("Error decoding MachineProviderConfig: %v", err)
+		glog.Errorf("%s: Error decoding MachineProviderConfig: %v", machine.Name, err)
 		return nil, err
 	}
 
@@ -423,17 +532,18 @@ func (a *Actuator) getMachineInstances(cluster *clusterv1.Cluster, machine *mach
 	}
 	client, err := a.awsClientBuilder(a.client, credentialsSecretName, machine.Namespace, region)
 	if err != nil {
-		glog.Errorf("Error getting EC2 client: %v", err)
-		return nil, fmt.Errorf("error getting EC2 client: %v", err)
+		errMsg := fmt.Sprintf("%s: Error getting EC2 client: %v", machine.Name, err)
+		glog.Errorf(errMsg)
+		return nil, fmt.Errorf(errMsg)
 	}
 
 	return getExistingInstances(machine, client)
 }
 
 // updateLoadBalancers adds a given machine instance to the load balancers specified in its provider config
-func (a *Actuator) updateLoadBalancers(client awsclient.Client, providerConfig *providerconfigv1.AWSMachineProviderConfig, instance *ec2.Instance) error {
+func (a *Actuator) updateLoadBalancers(client awsclient.Client, providerConfig *providerconfigv1.AWSMachineProviderConfig, instance *ec2.Instance, machineName string) error {
 	if len(providerConfig.LoadBalancers) == 0 {
-		glog.V(4).Infof("Instance %q has no load balancers configured. Skipping", *instance.InstanceId)
+		glog.V(4).Infof("%s: Instance %q has no load balancers configured. Skipping", machineName, *instance.InstanceId)
 		return nil
 	}
 	errs := []error{}
@@ -452,14 +562,14 @@ func (a *Actuator) updateLoadBalancers(client awsclient.Client, providerConfig *
 	if len(classicLoadBalancerNames) > 0 {
 		err := registerWithClassicLoadBalancers(client, classicLoadBalancerNames, instance)
 		if err != nil {
-			glog.Errorf("Failed to register classic load balancers: %v", err)
+			glog.Errorf("%s: Failed to register classic load balancers: %v", machineName, err)
 			errs = append(errs, err)
 		}
 	}
 	if len(networkLoadBalancerNames) > 0 {
 		err = registerWithNetworkLoadBalancers(client, networkLoadBalancerNames, instance)
 		if err != nil {
-			glog.Errorf("Failed to register network load balancers: %v", err)
+			glog.Errorf("%s: Failed to register network load balancers: %v", machineName, err)
 			errs = append(errs, err)
 		}
 	}
@@ -472,12 +582,12 @@ func (a *Actuator) updateLoadBalancers(client awsclient.Client, providerConfig *
 // updateStatus calculates the new machine status, checks if anything has changed, and updates if so.
 func (a *Actuator) updateStatus(machine *machinev1.Machine, instance *ec2.Instance) error {
 
-	glog.Info("Updating status")
+	glog.Infof("%s: Updating status", machine.Name)
 
 	// Starting with a fresh status as we assume full control of it here.
 	awsStatus := &providerconfigv1.AWSMachineProviderStatus{}
 	if err := a.codec.DecodeProviderStatus(machine.Status.ProviderStatus, awsStatus); err != nil {
-		glog.Errorf("Error decoding machine provider status: %v", err)
+		glog.Errorf("%s: Error decoding machine provider status: %v", machine.Name, err)
 		return err
 	}
 
@@ -516,7 +626,7 @@ func (a *Actuator) updateStatus(machine *machinev1.Machine, instance *ec2.Instan
 			})
 		}
 	}
-	glog.Info("finished calculating AWS status")
+	glog.Infof("%s: finished calculating AWS status", machine.Name)
 
 	awsStatus.Conditions = setAWSMachineProviderCondition(awsStatus.Conditions, providerconfigv1.MachineCreation, corev1.ConditionTrue, MachineCreationSucceeded, "machine successfully created", updateConditionIfReasonOrMessageChange)
 	// TODO(jchaloup): do we really need to update tis?
@@ -534,7 +644,7 @@ func (a *Actuator) updateStatus(machine *machinev1.Machine, instance *ec2.Instan
 	// attempting to update status until it hits a more permanent state. This will ensure
 	// we get a public IP populated more quickly.
 	if awsStatus.InstanceState != nil && *awsStatus.InstanceState == ec2.InstanceStateNamePending {
-		glog.Infof("Instance state still pending, returning an error to requeue")
+		glog.Infof("%s: Instance state still pending, returning an error to requeue", machine.Name)
 		return &clustererror.RequeueAfterError{RequeueAfter: requeueAfterSeconds * time.Second}
 	}
 	return nil
